# Copyright 2021 Tecnativa - Carlos Roca
# License AGPL-3.0 or later (https://www.gnu.org/licenses/agpl).

from odoo import http
from odoo.http import request
from odoo.addons.website.controllers.main import QueryURL
from odoo.addons.sale.controllers.variant import VariantController

from odoo.addons.website_sale.controllers.main import WebsiteSale
from odoo.addons.website_sale.controllers.main import TableCompute
import logging


_logger = logging.getLogger(__name__)

class WebsiteSaleInheritController(WebsiteSale):
    @http.route('/shop/products/autocomplete', type='json', auth='public', website=True)
    def products_autocomplete(self, term, options={}, **kwargs):
        
        ProductTemplate = request.env['product.template']
        
        pricelist = request.env["website"].get_current_website().get_current_pricelist()
        
        display_description = options.get('display_description', True)
        display_price = options.get('display_price', True)
        order = self._get_search_order(options)
        max_nb_chars = options.get('max_nb_chars', 999)

        category = options.get('category')
        attrib_values = options.get('attrib_values')

        domain = self._get_search_domain(term, category, attrib_values, display_description)
        
        products = ProductTemplate.search(
            domain,
            limit=min(20, options.get('limit', 5)),
            order=order
        )

        fields = ['id', 'name', 'website_url']
        if display_description:
            fields.append('description_sale')

        res = {
            'products': products.read(fields),
            'products_count': ProductTemplate.search_count(domain),
        }

        if display_description:
            for res_product in res['products']:
                desc = res_product['description_sale']
                if desc and len(desc) > max_nb_chars:
                    res_product['description_sale'] = "%s..." % desc[:(max_nb_chars - 3)]

        if display_price:
            FieldMonetary = request.env['ir.qweb.field.monetary']
            monetary_options = {
                'display_currency': request.website.get_current_pricelist().currency_id,
            }
            for res_product, product in zip(res['products'], products):
            	product_id, add_qty, has_distinct_price = product._get_cheapest_info(pricelist)
            	combination_info = product._get_combination_info(product_id=product_id, add_qty=add_qty, pricelist=pricelist)
            	res_product.update(combination_info)
            	res_product['list_price'] = FieldMonetary.value_to_html(res_product['list_price'], monetary_options)
            	res_product['price'] = FieldMonetary.value_to_html(res_product['price'], monetary_options)

        return res
    
    def _get_pricelist_context(self):
        pricelist_context = dict(request.env.context)
        pricelist = False
        if not pricelist_context.get('pricelist'):
            pricelist = request.website.get_current_pricelist()
            pricelist_context['pricelist'] = pricelist.id
        else:
            pricelist = request.env['product.pricelist'].browse(pricelist_context['pricelist'])

        return pricelist_context, pricelist

    def _get_search_order(self, post):
        # OrderBy will be parsed in orm and so no direct sql injection
        # id is added to be sure that order is a unique sort key
        order = post.get('order') or 'website_sequence ASC'
        return 'is_published desc, %s, id desc' % order

    

    def sitemap_shop(env, rule, qs):
        if not qs or qs.lower() in '/shop':
            yield {'loc': '/shop'}

        Category = env['product.public.category']
        dom = sitemap_qs2dom(qs, '/shop/category', Category._rec_name)
        dom += env['website'].get_current_website().website_domain()
        for cat in Category.search(dom):
            loc = '/shop/category/%s' % slug(cat)
            if not qs or qs.lower() in loc:
                yield {'loc': loc}
        
    

class WebsiteSaleVariantController(VariantController):
    @http.route(
        ["/sale/get_combination_info_minimal_price"],
        type="json",
        auth="public",
        methods=["POST"],
        website=True,
    )
    def get_combination_info_minimal_price(self, product_template_ids, **kw):
        """Special route to use website logic in get_combination_info override.
        This route is called in JS by appending _website to the base route.
        """
        res = []
        templates = request.env["product.template"].sudo().browse(product_template_ids)
        pricelist = request.env["website"].get_current_website().get_current_pricelist()
        for template in templates.filtered(lambda t: t.is_published):
            product_id, add_qty, has_distinct_price = template._get_cheapest_info(
                pricelist
            )
            combination = template._get_combination_info(
                product_id=product_id, add_qty=add_qty, pricelist=pricelist
            )
            res.append(
                {
                    "id": template.id,
                    "price": combination.get("price"),
                    "distinct_prices": has_distinct_price,
                    "list_price":combination['list_price'], #GeminateCS
                    "currency": {
                        "position": template.currency_id.position,
                        "symbol": template.currency_id.symbol,
                    },
                }
            )
        return res

    @http.route(
        ["/sale/get_combination_info_pricelist_atributes"],
        type="json",
        auth="public",
        website=True,
    )
    def get_combination_info_pricelist_atributes(self, product_id, **kwargs):
        """Special route to use website logic in get_combination_info override.
        This route is called in JS by appending _website to the base route.
        """
        product = request.env["product.product"].browse(product_id)
        pricelist = request.env["website"].get_current_website().get_current_pricelist()
        # Getting all min_quantity of the current product to compute the possible
        # price scale.
        qty_list = request.env["product.pricelist.item"].search(
            [
                "|",
                ("product_id", "=", product.id),
                "|",
                ("product_tmpl_id", "=", product.product_tmpl_id.id),
                (
                    "categ_id",
                    "in",
                    list(map(int, product.categ_id.parent_path.split("/")[0:-1])),
                ),
                ("min_quantity", ">", 0),
            ]
        )
        qty_list = sorted(set(qty_list.mapped("min_quantity")))
        res = []
        ctx = dict(request.env.context, pricelist=pricelist.id, quantity=0)
        last_price = product.with_context(ctx).price
        for min_qty in qty_list:
            ctx["quantity"] = min_qty
            new_price = product.with_context(ctx).price
            if new_price != last_price:
                res.append(
                    {
                        "min_qty": min_qty,
                        "price": new_price,
                        "currency": {
                            "position": product.currency_id.position,
                            "symbol": product.currency_id.symbol,
                        },
                    }
                )
                last_price = new_price
        return (res, product.uom_name)
